;**************************************************************
;keypad_test
;Tests keypad, writing key pressed to lcd display on 
;Derbot Hand Controller. 
;The lcd controller is S6A0069, whose initialisation is applied.
;TJW rev. 20.7.06							Tested 20.7.06
;***************************************************************
;Clock is 1MHz approx
;Configuration Word: RC oscillator, WDT off, power-up timer on, 
;			   brown-out off, LVP off, all code protect on.
;								
;   Port A      Port B      	Port C
;   ------      ------          ------
;0	lcd bus 2	not used		Display RS
;1	lcd bus 3	keypad col 3	Display E
;2	lcd bus 4	keypad col 2	Interrupt op
;3	lcd bus 5	keypad col 1	SCL
;4	lcd bus 6	keypad row 4	SDA
;5	lcd bus 7	keypad row 3	Display R/W
;6	    -		keypad row 2 	lcd bus 0
;7	    -		keypad row 1   	lcd bus 1

	list 	p=16F873A
	#include p16f873A.inc

;Specify RAM
delcntr1 equ	20	;used in delays
delcntr2 equ	21
temp	 equ	22  ;a temp location, to be used only in consecutive instructions
lcd_op	 equ	23	;holds word to be sent to display
pointer  equ    24  ;points to Character string Tables
kpad_pat equ	25	;holds pattern of keypad
kpad_add equ	26  ;holds kpad address to lu table
kpad_char equ	27	;holds most recent character from keypad
flags	 equ	28	;bit 0 holds RS state during busy_check SR
			
;Specify some port bits
;Port C
lcd_RS	equ	0	;0 for instruction, 1 for character
lcd_E	equ 1
lcd_RW	equ	5
;Port A
lcd_busy equ 5

	org 00
	goto main
	org 04		; 
	goto kpad_to_lcd
;Initialise
main bcf	STATUS,RP1
	bsf		STATUS,RP0 	; RP1-RP0 01 -   1
	movlw	B'00000000'	;  A & C  
	movwf	TRISA		
	movwf	TRISC		
	movlw	B'00000110'	;   -  /
	movwf	ADCON1
	movlw	B'11110000'	;   7-4- , 3-0- 
	movwf	TRISB		;	
	bcf		STATUS,RP0 	;RP1-RP0 00 -   0
;
;initialise display 
		bcf	PORTB,0		;diagnostic, used in busy_check ***
		call delay200		;wait for power supply to settle
		bcf	PORTC,lcd_RS	;set control lines
		bcf	PORTC,lcd_RW
;     
;    .    .
;Relies first on automatic initialisation, as described in data sheet
;now send four command words. Busy Flag not used. 
		movlw B'00001111'	;  , ,  2 (00111100)
		movwf PORTA			;first command word, Function Set, rotated by 2 bits
		bsf	PORTC,lcd_E
		bcf	PORTC,lcd_E
		call delay1			;wait at least 39us
;
		movlw B'00000011'	;next command, Display ON/OFF(00001111)
		movwf PORTA
		bsf	PORTC,6
		bsf	PORTC,7
		bsf	PORTC,lcd_E		
		bcf	PORTC,lcd_E
		call delay1			;wait at least 39us
;
		movlw B'00000000'	;next command, Display Clear(00000001)
		movwf PORTA
		bcf	PORTC,7
		bsf	PORTC,6
		bsf	PORTC,lcd_E		
		bcf	PORTC,lcd_E
		call delay1		
		call delay1			;wait at least 1.53ms
;
		movlw B'00000001'	;next command, Entry Mode Set (00000110)
		movwf PORTA
		bsf	PORTC,7
		bcf	PORTC,6
		bsf	PORTC,lcd_E		
		bcf	PORTC,lcd_E
		call delay1	
;
;	
		clrf	PORTB	;    ()
		bcf		INTCON,RBIF	
		bsf		INTCON,RBIE ;  
		bsf		INTCON,GIE
loop	goto	loop		;    

;***********************************************************
;   () 
;***********************************************************
;   B(4  ),      -
;   ,    kpad_char.       . 
kpad_to_lcd call 	kpad_rd
;     ,      .
		call 	kp_code_conv	
;     
		movf	kpad_char,0 ;  kpad_char  W (  )
		movwf	lcd_op
		bsf		PORTC,lcd_RS ;   
		call	lcd_write
;   
rel_test call 	kpad_rd		
		movf	kpad_pat,0	; kpad_pat  W
		andlw	0fe			;     (. 0fe  W. 11=1,  0)
		sublw	0fe			;  ( W  )
;bsf STATUS, Z; !!!					
		btfss	STATUS,Z	; 2 .  0,     .  
							;   0( ).  1 =>  0.( )
		goto 	rel_test
		bcf		INTCON,RBIF ;  
		retfie
;	
;***********************************************************
;
;***********************************************************
; ,     kpad_pat 
;   
kpad_rd	movf	PORTB,W	; PORTB,   W
		andlw	B'11110000' ;   (. B'11110000  W)
		movwf	kpad_pat
		bsf		STATUS,RP0 ;      - 
		movlw	B'00001110'
		movwf	TRISB
		bcf		STATUS,RP0
		movlw	00
		movwf	PORTB	;    
		movf	PORTB,W	; potrb -  
		andlw	B'00001110' ;  ("" 1  1=1,  0)
		iorwf	kpad_pat,1	;. (""0  0=0,  1. .  kpad_pat)
	;movlw   B'01110110' ; !!!
	;movwf  	kpad_pat	; !!!( 1)
;  
		bsf		STATUS,RP0 
		movlw	B'11110000' ;-, -
		movwf	TRISB
		bcf		STATUS,RP0
		clrf	PORTB	;   
		return

;   kpad_pat  ASCII character  .   
;( kpad_add),       . 
;  ,   kpad_char
kp_code_conv bcf STATUS,C 	;  - 0- 
		rrf		kpad_pat,1	;  0,   
							;     (.  kpad_pat)
							;..  kpad_pat,0  0,  =0( ). kpad_pat,0  1,  =1
		clrf	kpad_add
;
;     ,     ( ).      ,       col_find  keypad_op
;
; 
		btfsc	kpad_pat,6 ;     0
		goto	kp1
		goto	col_find	; 1 , kpad_add   
kp1		btfsc	kpad_pat,5
		goto	kp2
		movlw	B'00000100'	;  2
		iorwf	kpad_add,1	;   (00=0,  1)
		goto	col_find
kp2		btfsc	kpad_pat,4
		goto	kp3
		movlw	B'00001000'	;  3
		iorwf	kpad_add,1	;   (00=0,  1)
		goto	col_find
kp3		btfsc	kpad_pat,3
		goto	kp4
		movlw	B'00001100'	;  4
		iorwf	kpad_add,1	;   (00=0,  1)
		goto	col_find
kp4		movlw	D'16'		;  ,      ('E')
		;    kpad_add   0. => 1    
		goto keypad_op
; 
col_find btfsc	kpad_pat,2
		goto	cf1
		goto    keypad_op	; 1, kpad_add   
cf1		btfsc	kpad_pat,1
		goto	cf2
		movlw	B'00000001'	; 2
		iorwf	kpad_add,1	;   (00=0,  1)
		goto    keypad_op
;   3
cf2		movlw	B'00000010'			
		iorwf	kpad_add,1	;  	
keypad_op movf	kpad_add,0	;  W   
		call	kp_table
		movwf	kpad_char	; 
		return
;
;,    ,   ,    .
;  ASCII     . 
kp_table addwf PCL,1
		retlw	'1'		; 1  ASCII
		retlw	'2'
		retlw	'3'
		retlw	'A'		;Error code
		retlw	'4'		; 2
		retlw	'5'
		retlw	'6'
		retlw	'B'		;Error code
		retlw	'7'		; 3
		retlw	'8'
		retlw	'9'
		retlw	'C'		;Error code
		retlw	'*'		; 4
		retlw	'0'
		retlw	'#'
		retlw	'D'		;Error code
		retlw	'E'		;Error code
;
;         ,   lcd_op  .
;RS       ,   . 
lcd_write call busy_check
		bcf	PORTC,lcd_RW
		bcf	STATUS,C
		rrf	lcd_op,1	;!  ,        
		bcf	PORTC,6		;!   0 
		btfsc STATUS,C	; 0   
		bsf	PORTC,6		; 
		bcf	STATUS,C	; 
		rrf	lcd_op,1	;     
		bcf	PORTC,7		;!   1 
		btfsc STATUS,C
		bsf	PORTC,7
		movf lcd_op,0	;   W
		movwf PORTA
		bsf	PORTC,lcd_E
		bcf	PORTC,lcd_E
		return
;      
busy_check bsf STATUS,RP0 	;  1
		movlw B'00111111'	;      
		movwf TRISA
		bcf	STATUS,RP0
 		bcf	flags,0			
		btfsc PORTC,lcd_RS	;  RS   flags, 0
		bsf	flags,0		
		bcf	PORTC,lcd_RS 	;   
		bsf	PORTC,lcd_RW	;  
busy_loop bcf PORTC,lcd_E	
		bsf	PORTC,lcd_E
		btfsc PORTA,lcd_busy ;  , ,     
		goto busy_loop		 ; 1,    		
		bcf	PORTC,lcd_E	
		bsf	STATUS,RP0 		;    1
		movlw B'00000000'	;       
		movwf TRISA
		bcf	STATUS,RP0
		bcf	PORTC,lcd_RS	;   
		btfsc flags,0		;  RS
		bsf	PORTC,lcd_RS	
		return
;
;    4  
;   100us
delay100u  movlw 	D'6' ;6   16us
        movwf  	delcntr1
;4   c   16us
del1001    nop				;1  
        decfsz 	delcntr1,1	;1  ,    
        goto 	del1001		;2  
        return

;   1ms
delay1  movlw 	D'62'	;62   16us
        movwf  	delcntr1
del1    nop				   ;4   c   16us
        decfsz 	delcntr1,1 ; 1,   0,   . 
        goto 	del1
        return
;
;   5ms 
delay5  movlw 	D'250'	;250  
        movwf  	delcntr1
del51   nop				;5   c   20us
 		nop
        decfsz 	delcntr1,1
        goto 	del51
        return
;
;   200ms, 200   delay1
;200ms(200  delay1) + 200*20us( 5 . )=204ms 
delay200  movlw D'200'
        movwf  	delcntr2
del2    call 	delay1		;2  
        decfsz  delcntr2,1	;1  
        goto 	del2		;2  
        return  
;
	end
